import com.ms.wfc.app.Registry;
import com.ms.wfc.app.RegistryKey;
import java.util.StringTokenizer;
/**
 * This class can take a variable number of parameters on the command
 * line. Program execution begins with the main() method. The class
 * constructor is not invoked unless an object of type 'Class1'
 * created in the main() method.
 */
public class SetDefLp
{
	/**
	 * The main entry point for the application. 
	 *
	 * @param args Array of parameters passed to the application
	 * via the command line.
	 */
	public static void main (String[] args)
	{
		boolean
			readonly = true,		// Flag set for read only capability
			readwrite = false;		// Flag set for read/write capability
		int 
			count,
			i;						// Counter, holds position in lpList of new printer 
		RegistryKey
			currDefLpKey,			// Key pointing to currently defined default printer
			storedDefLpKey,			// Key pointing to stored info about user's default printer
			lpListKey;				// Key pointing to names of defined printers for user
		String[]
			tokens = new String[3],	// Array of tokens from stored default printer info
			lpList;					// Array containing list of user's defined printers
		String
			storedDefLpInfo,		// String of info for stored default printer
			currDefLpInfo,			// String of info for current default printer
			newDefLpInfo;			// String of info for new default printer
		StringTokenizer
			storedDefLpTokens;		// Tokenized info for stored default printer

		if (args.length != 1){				// Wrong usage of command, print error
			System.out.println("Usage: SetDefLp name_of_printer |\n"+
							   "       SetDefLp -r (restore default)");
		}
		else {								// Only one argument
			
			// Open registry keys
			currDefLpKey = Registry.CURRENT_USER.getSubKey
				("Software\\Microsoft\\Windows NT\\CurrentVersion\\Windows",readwrite);
			storedDefLpKey = Registry.CURRENT_USER.getSubKey("Printers",readwrite);
			lpListKey = Registry.CURRENT_USER.getSubKey
				("Software\\Microsoft\\Windows NT\\CurrentVersion\\Devices",readonly);
			
			// Check what the argument was
			if (args[0].equals("-r")) {		// Restore default printer
			
				// Restore stored default printer info if valid
				storedDefLpInfo = storedDefLpKey.getValue("DeviceOld").toString();
				storedDefLpTokens = new StringTokenizer(storedDefLpInfo,",");
				if (storedDefLpTokens.countTokens() == 3) {
					for (i=0; i < 3; i++)
						tokens[i] = storedDefLpTokens.nextToken();
					if (lpListKey.getValue(tokens[0]).equals(tokens[1]+","+tokens[2]))
						currDefLpKey.setValue("Device",storedDefLpInfo);
					else
						System.out.println("Invalid stored default printer");
				}
				else
					System.out.println("Invalid stored default printer");
			}
			else {							// Set new default printer, save current

				// Get list of user's currently defined printers
				lpList = lpListKey.getValueNames();
			
				// Check to see if new default printer is defined on machine
				for (i=0; i < lpList.length; i++)
					if (lpList[i].startsWith("\\"))				// Skip if not network printer
						if (lpList[i].endsWith("\\"+args[0]))	// Found
							break;
			
				if (i == lpList.length){	// Network printer not found, print error
					System.out.println("Undefined Network Printer: "+args[0]);
				}
				else {
					
					// Store current default printer info if necessary
					currDefLpInfo = currDefLpKey.getValue("Device").toString();
					if (storedDefLpKey.getValue("DeviceOld").equals(currDefLpInfo) == false)
						storedDefLpKey.setValue("DeviceOld",currDefLpInfo);
					
					// Set new default printer if necessary
					newDefLpInfo = lpList[i]+","+lpListKey.getValue(lpList[i]);
					if (newDefLpInfo.equals(currDefLpInfo) == false)
						currDefLpKey.setValue("Device",newDefLpInfo);
					
				}
				
			}
			// Clean up by closing registry keys
			currDefLpKey.close();
			storedDefLpKey.close();
			lpListKey.close();
		}
	}
}
